% Function to determine moment-angle curves from raw angle and moment data based on 1-D interpolation
% written by A. Pennycott, February 2023

function [phi_knee_S_stand_interp, phi_knee_P_stand_interp, M_knee_S_stand_mean, M_knee_P_stand_mean, ...
    phi_knee_S_sit_interp, phi_knee_P_sit_interp, M_knee_S_sit_mean, M_knee_P_sit_mean] = CalcAverageCurves(partitionedData)

% standing data
phi_knee_S_stand_interp = cell(size(partitionedData));
phi_knee_P_stand_interp = cell(size(partitionedData));
M_knee_S_stand_interp = cell(size(partitionedData));
M_knee_P_stand_interp = cell(size(partitionedData));
M_knee_S_stand_mean = cell(size(partitionedData));
M_knee_P_stand_mean = cell(size(partitionedData));

for i = 1:size(partitionedData, 1)

    for j = 1:2
        X_min_S_stand = 1e5;
        X_max_S_stand = -1e5;

        X_min_P_stand = 1e5;
        X_max_P_stand = -1e5;

        M_S_stand = cell(length(partitionedData{i, j}.stand.phi_knee_S));
        M_P_stand = cell(length(partitionedData{i, j}.stand.phi_knee_S));

        for k = 1:length(partitionedData{i, j}.stand.phi_knee_S)
            X_min_S_stand_temp = min(partitionedData{i, j}.stand.phi_knee_S{k});
            X_min_P_stand_temp = min(partitionedData{i, j}.stand.phi_knee_P{k});
            X_max_S_stand_temp = max(partitionedData{i, j}.stand.phi_knee_S{k});
            X_max_P_stand_temp = max(partitionedData{i, j}.stand.phi_knee_P{k});

            if X_min_S_stand_temp < X_min_S_stand
                X_min_S_stand = X_min_S_stand_temp;
            end

            if X_min_P_stand_temp < X_min_P_stand
                X_min_P_stand = X_min_P_stand_temp;
            end

            if X_max_S_stand_temp > X_max_S_stand
                X_max_S_stand = X_max_S_stand_temp;
            end

            if X_max_P_stand_temp > X_max_P_stand
                X_max_P_stand = X_max_P_stand_temp;
            end

            m_S_stand = [partitionedData{i, j}.stand.phi_knee_S{k} partitionedData{i, j}.stand.M_knee_S{k}];
            m_P_stand = [partitionedData{i, j}.stand.phi_knee_P{k} partitionedData{i, j}.stand.M_knee_P{k}];

            [~, index_S_stand] = sort(m_S_stand(:, 1), 1);
            [~, index_P_stand] = sort(m_P_stand(:, 1), 1);

            m_S_stand = m_S_stand(index_S_stand, :);
            m_P_stand = m_P_stand(index_P_stand, :);

            [~, index_S_stand] = unique(m_S_stand(:, 1));
            [~, index_P_stand] = unique(m_P_stand(:, 1));

            M_S_stand{k} = m_S_stand(index_S_stand, :);
            M_P_stand{k} = m_P_stand(index_P_stand, :);

            phi_knee_S_stand_interp{i, j} = (X_min_S_stand:(X_max_S_stand - X_min_S_stand) / 100:X_max_S_stand);
            phi_knee_P_stand_interp{i, j} = (X_min_P_stand:(X_max_P_stand - X_min_P_stand) / 100:X_max_P_stand);
        end

        for k = 1:length(partitionedData{i, j}.stand.phi_knee_S)
            M_knee_S_stand_interp{i, j}(k, :) = interp1((M_S_stand{k}(:,1))', (M_S_stand{k}(:,2))', phi_knee_S_stand_interp{i, j});
            M_knee_P_stand_interp{i, j}(k, :) = interp1((M_P_stand{k}(:,1))', (M_P_stand{k}(:,2))', phi_knee_P_stand_interp{i, j});
        end

        M_knee_S_stand_mean{i, j} = mean(M_knee_S_stand_interp{i, j}, 1);
        M_knee_P_stand_mean{i, j} = mean(M_knee_P_stand_interp{i, j}, 1);
    end
end

% sitting data
phi_knee_S_sit_interp = cell(size(partitionedData));
phi_knee_P_sit_interp = cell(size(partitionedData));
M_knee_S_sit_interp = cell(size(partitionedData));
M_knee_P_sit_interp = cell(size(partitionedData));
M_knee_S_sit_mean = cell(size(partitionedData));
M_knee_P_sit_mean = cell(size(partitionedData));

for i = 1:size(partitionedData, 1)

    for j = 1:2
        X_min_S_sit = 1e5;
        X_max_S_sit = -1e5;

        X_min_P_sit = 1e5;
        X_max_P_sit = -1e5;

        M_S_sit = cell(length(partitionedData{i, j}.sit.phi_knee_S));
        M_P_sit = cell(length(partitionedData{i, j}.sit.phi_knee_S));

        for k = 1:length(partitionedData{i, j}.sit.phi_knee_S)
            X_min_S_sit_temp = min(partitionedData{i, j}.sit.phi_knee_S{k});
            X_min_P_sit_temp = min(partitionedData{i, j}.sit.phi_knee_P{k});
            X_max_S_sit_temp = max(partitionedData{i, j}.sit.phi_knee_S{k});
            X_max_P_sit_temp = max(partitionedData{i, j}.sit.phi_knee_P{k});

            if X_min_S_sit_temp < X_min_S_sit
                X_min_S_sit = X_min_S_sit_temp;
            end

            if X_min_P_sit_temp < X_min_P_sit
                X_min_P_sit = X_min_P_sit_temp;
            end

            if X_max_S_sit_temp > X_max_S_sit
                X_max_S_sit = X_max_S_sit_temp;
            end

            if X_max_P_sit_temp > X_max_P_sit
                X_max_P_sit = X_max_P_sit_temp;
            end

            m_S_sit = [partitionedData{i, j}.sit.phi_knee_S{k} partitionedData{i, j}.sit.M_knee_S{k}];
            m_P_sit = [partitionedData{i, j}.sit.phi_knee_P{k} partitionedData{i, j}.sit.M_knee_P{k}];

            [~, index_S_sit] = sort(m_S_sit(:, 1), 1);
            [~, index_P_sit] = sort(m_P_sit(:, 1), 1);

            m_S_sit = m_S_sit(index_S_sit, :);
            m_P_sit = m_P_sit(index_P_sit, :);

            [~, index_S_sit] = unique(m_S_sit(:, 1));
            [~, index_P_sit] = unique(m_P_sit(:, 1));

            M_S_sit{k} = m_S_sit(index_S_sit, :);
            M_P_sit{k} = m_P_sit(index_P_sit, :);

            phi_knee_S_sit_interp{i, j} = (X_min_S_sit:(X_max_S_sit - X_min_S_sit) / 100:X_max_S_sit);
            phi_knee_P_sit_interp{i, j} = (X_min_P_sit:(X_max_P_sit - X_min_P_sit) / 100:X_max_P_sit);
        end

        for k = 1:length(partitionedData{i, j}.sit.phi_knee_S)
            M_knee_S_sit_interp{i, j}(k, :) = interp1((M_S_sit{k}(:,1))', (M_S_sit{k}(:,2))', phi_knee_S_sit_interp{i, j});
            M_knee_P_sit_interp{i, j}(k, :) = interp1((M_P_sit{k}(:,1))', (M_P_sit{k}(:,2))', phi_knee_P_sit_interp{i, j});
        end

        M_knee_S_sit_mean{i, j} = mean(M_knee_S_sit_interp{i, j}, 1);
        M_knee_P_sit_mean{i, j} = mean(M_knee_P_sit_interp{i, j}, 1);
    end
end
