% Function to partition data into standing up and sitting down phases based on visual inspection of knee angle and moment data
% written by A. Pennycott, February 2023

function partitionedData = CalcPartitionedData(processedData)

% preallocation of return data
partitionedData = cell(size(processedData));

% Subject 1
partitionedData{1, 1}.stand.StartStop = [0 3.6; 16.7 21.3; 33.5 38.3; 45.8 50.3; 60.0 64.9];
partitionedData{1, 1}.sit.StartStop =   [3.6 14.4; 21.3 28.7; 38.3 45.8; 50.3 60.0];

partitionedData{1, 2}.stand.StartStop = [88.9 91.4; 102.7 106.9];
partitionedData{1, 2}.sit.StartStop =   [91.4 100.2; 106.9 117.8];

% Subject 2
partitionedData{2, 1}.stand.StartStop = [1.4 3.3;7.2 9.2; 12.5 14.4; 17.3 19.11; 22.2 24.6];
partitionedData{2, 1}.sit.StartStop =   [3.9 6.3; 9.6 11.8; 14.7 16.9; 19.6 21.7; 25.5 28.3];

partitionedData{2, 2}.stand.StartStop = [15.9 19.9; 30.9 34.2; 38.8 41.5; 44.4 49.0];
partitionedData{2, 2}.sit.StartStop =   [19.9 30.0; 34.2 38.0; 41.5 44.4];

% Subject 3
partitionedData{3, 1}.stand.StartStop = [0.8 4.8;21.2 25.0; 28.9 33.6; 41.0 45.2; 50.4 54.5];
partitionedData{3, 1}.sit.StartStop =   [4.8 10.2; 25.0 28.0; 33.6 36.7; 45.2 48.6;54.5 56.9];

partitionedData{3, 2}.stand.StartStop = [0.5 6.3; 22.1 27.1; 36.0 40.0; 58.1 62.0; 68.5 72.5];
partitionedData{3, 2}.sit.StartStop =   [11.0 17.7; 29.4 33.1; 40.1 47.2; 62.0 68.5; 72.5 76.3];

% Subject 4
partitionedData{4, 1}.stand.StartStop = [0 2.1; 5.1 7.3; 11.6 13.9; 17.4 19.7; 24.9 26.9];
partitionedData{4, 1}.sit.StartStop =   [2.1 4.1; 7.8 10.6; 14.3 16.8; 20.1 22.7; 27.9 30.5];

partitionedData{4, 2}.stand.StartStop = [0.4 2.8; 17.3 20.4; 27.0 30.3; 39.7 42.6; 49.3 52.5];
partitionedData{4, 2}.sit.StartStop =   [7.9 14.5; 21.4 25.7; 31.2 36.4; 43.3 47.1; 53.4 58.0];

% Subject 5
partitionedData{5, 1}.stand.StartStop = [0.4 3.3; 8.3 11.5; 15.1 18.3; 21.8 23.7;27.1 31.2];
partitionedData{5, 1}.sit.StartStop =   [5.4 8.3; 11.5 15.1; 18.3 21.8; 24.6 27.1; 31.2 34.0];

partitionedData{5, 2}.stand.StartStop = [2.7 6.2; 18.7 22.1; 29.4 32.1;47.6 52.0; 61.4 64.6];
partitionedData{5, 2}.sit.StartStop =   [10.8 16.9; 22.1 27.7; 36.1 42.1; 53.8 58.8; 64.5 69.3];

% Subject 6
partitionedData{6, 1}.stand.StartStop = [1.2 3.3; 7.4 9.1; 11.5 13.1; 17.4 18.8; 21.2 22.4];
partitionedData{6, 1}.sit.StartStop =   [3.3 6.7; 9.3 11.0; 14.9 17.0; 19.4 20.8; 22.9 24.4];

partitionedData{6, 2}.stand.StartStop = [0.4 2.6; 5.9 7.9; 19.4 21.1; 25.5 26.9];
partitionedData{6, 2}.sit.StartStop =   [2.6 5.4; 7.9 10.4; 15.7 18.0; 22.2 24.9; 27.3 29.4];

% Subject 7
partitionedData{7, 1}.stand.StartStop = [1.6 4.5; 7.4 10.2; 13.1 15.7;18.6 21.0; 24.1 26.4];
partitionedData{7, 1}.sit.StartStop =   [4.5 7.4; 10.2 12.8; 15.7 18.2; 21.0 23.5; 26.4 28.7];

partitionedData{7, 2}.stand.StartStop = [1.6 4.1; 10.9 14.3; 19.1 22.7; 27.8 31.0; 35.6 38.8];
partitionedData{7, 2}.sit.StartStop =   [6.4 10.9; 15.9 19.1; 23.8 27.2; 31.6 35.3; 39.1 42.1];

% divide knee angle and moments according to stop and start indices
for i = 1:size(partitionedData, 1)
    for j = 1:2 
        % standing data
        for k = 1:length(partitionedData{i, j}.stand.StartStop)
            a = find(processedData{i, j}.time == partitionedData{i, j}.stand.StartStop(k, 1));
            b = find(processedData{i, j}.time == partitionedData{i, j}.stand.StartStop(k, 2));
            if (isempty(a))
                a = 1;
            end
            if (isempty(b))
                b = 1;
            end
            partitionedData{i, j}.stand.phi_knee_S{k} = processedData{i, j}.phi_knee_S(a:b);
            partitionedData{i, j}.stand.phi_knee_P{k} = processedData{i, j}.phi_knee_P(a:b);
            partitionedData{i, j}.stand.M_knee_S{k} = processedData{i, j}.M_knee_S(a:b);
            partitionedData{i, j}.stand.M_knee_P{k} = processedData{i, j}.M_knee_P(a:b);
        end

        % sitting data
        for k = 1:length(partitionedData{i, j}.sit.StartStop)
            a = find(processedData{i, j}.time == partitionedData{i, j}.sit.StartStop(k, 1));
            b = find(processedData{i, j}.time == partitionedData{i, j}.sit.StartStop(k, 2));
            if (isempty(a))
                a = 1;
            end
            if (isempty(b))
                b = 1;
            end
            partitionedData{i, j}.sit.phi_knee_S{k} = processedData{i, j}.phi_knee_S(a:b);
            partitionedData{i, j}.sit.phi_knee_P{k} = processedData{i, j}.phi_knee_P(a:b);
            partitionedData{i, j}.sit.M_knee_S{k} = processedData{i, j}.M_knee_S(a:b);
            partitionedData{i, j}.sit.M_knee_P{k} = processedData{i, j}.M_knee_P(a:b);
        end
    end
end
