% Function to determine coordinate system for each subject and condition
% Written by Simon van der Helm, April 2019

function X = ExtractCoordinateSystem(isubject, folderContents, indices_matfiles)

iStatic = find(and(startsWith(folderContents.name,'static'),folderContents.isdir==0)); % Check if a static test was done

if ~isempty(iStatic) % If the static file exists
    static = load([folderContents.folder{iStatic(1)},'\',folderContents.name{iStatic(1)}]); % Load the static file for local coordinate systems
else %only true for subject 2
    static = load([folderContents.folder{indices_matfiles(1)},'\',folderContents.name{indices_matfiles(1)}]); % Load the first data file for local coordinate systems
end

statField = fieldnames(static); % Find the fieldnames in the static structure

%For subject 5, the LASIS marker was not placed. See below for another
%exception
if isubject == '5'% used to be coded as  isubject == 8. with isubject being the number of the file in the folder, which is sensitive to folder structure % For the fifth subject the LASIS marker was not placed
    static.(statField{1}).Trajectories.Labeled = estimateIAS(static.(statField{1}).Trajectories.Labeled);
end

coordSys = MakeLocalCoord(static.(statField{1}).Trajectories.Labeled,'All'); % Make the local coordinate systems
X = localCoord(static.(statField{1}).Trajectories.Labeled,coordSys); % Construct the Local positions of the markers

function [data] = estimateIAS(data)

R_IPS = data.Data(ismember(data.Labels,'R-IPS'),1:3,:); % Right Superior Posterior Iliac Spine
L_IPS = data.Data(ismember(data.Labels,'L-IPS'),1:3,:); % Left Superior Posterior Iliac Spine
M_IPS = data.Data(ismember(data.Labels,'M-IPS'),1:3,:); % Middle Superior Posterior Iliac Spine

if ~ismember('L-IAS',data.Labels) && ismember('R-IAS',data.Labels) % Estimate Left Superior Anterior Iliac Spine
    R_IAS = data.Data(ismember(data.Labels,'R-IAS'),1:3,:); % Right Superior Anterior Iliac Spine
    L_IAS = zeros(1,size(data.Data,2),size(R_IAS,3));
    for i = 1:size(R_IAS,3) % For each instance
        n = squeeze(L_IPS(:,:,i)-R_IPS(:,:,i)); % Vector from Left to Right
        n = n/norm(n); % Normalize
        v = squeeze(R_IAS(:,:,i)-M_IPS(:,:,i)); % Vector from Right Superior Anterior Iliac Spine to the middle
        d = dot(v,n); % Find the projection of v on n
        L_IAS(1,1:3,i) = R_IAS(:,:,i)-2*d*n; % Substract the projection twice from the Right Superior Anterior Iliac Spine to find Left Superior Iliac Spine
    end
    data.Data(end+1,:,:) = L_IAS; % Save
    data.Labels{end+1} = 'L-IAS';
    
elseif ismember('L-IAS',data.Labels) && ~ismember('R-IAS',data.Labels) % Estimate Right Superior Anterior Iliac Spine
    L_IAS = data.Data(ismember(data.Labels,'L-IAS'),1:3,:); % Left Superior Anterior Iliac Spine
    R_IAS = zeros(1,size(data.Data,2),size(L_IAS,3));
    for i = 1:size(L_IAS,3) % For each instance
        n = squeeze(R_IPS(:,:,i)-L_IPS(:,:,i)); % Vector from Right to Left
        n = n/norm(n); % Normalize
        v = squeeze(L_IAS(:,:,i)-M_IPS(:,:,i)); % Vector from LASIS to Middle
        d = dot(v,n); % Find projection of v on n
        R_IAS(1,1:3,i) = L_IAS(:,:,i)-2*d*n; % Substract the projection twice from LASIS to find RASIS
    end
    data.Data(end+1,:,:) = R_IAS; % Save
    data.Labels{end+1} = 'R-IAS';
end
