% Function for plotting the knee moment versus knee angle for each subject across the various experimental conditions
% Written by A. Pennycott, February 2023

function PlotAverageCurves(phi_knee_S_stand_interp, phi_knee_P_stand_interp, M_knee_S_stand_mean, M_knee_P_stand_mean, ...
    phi_knee_S_sit_interp, phi_knee_P_sit_interp, M_knee_S_sit_mean, M_knee_P_sit_mean)

% standing plot
figure
for i = 1:size(phi_knee_S_stand_interp, 1)
    subplot(3,3,i);
    hold on;
    title(['Mean moment vs. angle during standing up: S', num2str(i)]);
    plot(phi_knee_S_stand_interp{i, 2} * 180 / pi, -M_knee_S_stand_mean{i, 2}, 'k');
    plot(phi_knee_P_stand_interp{i, 2} * 180 / pi, -M_knee_P_stand_mean{i, 2}, 'm');
    plot(phi_knee_S_stand_interp{i, 1} * 180 / pi, -M_knee_S_stand_mean{i, 1}, 'b');
    plot(phi_knee_P_stand_interp{i, 1} * 180 / pi, -M_knee_P_stand_mean{i, 1}, 'r');
    xlabel(['Knee angle (deg)']);
    ylabel('Knee moment (Nm)');
    xticks([0,20,40,60,80,100]);
    yticks([0,50,100,150]);
    xlim([0, 100]);
    ylim([-20, 150]);
end
lh = legend('Sound Leg with Support','Prosthetic Leg with Support','Sound Leg w/o Support','Prosthetic Leg w/o Support');
set(lh, 'FontSize', 12);
set(lh, 'Position', [0.4227    0.1658    0.1930    0.1225]);
set(gcf, 'Position', get(0, 'Screensize'));
%saveas(gcf, 'MeanTorqueCurvesStand', 'epsc');

% sitting plot
figure
for i = 1:size(phi_knee_S_sit_interp, 1)
    subplot(3,3,i);
    hold on;
    title(['Mean moment vs. angle during sitting down: S', num2str(i)]);
    plot(phi_knee_S_sit_interp{i, 2} * 180 / pi, -M_knee_S_sit_mean{i, 2}, 'k');
    plot(phi_knee_P_sit_interp{i, 2} * 180 / pi, -M_knee_P_sit_mean{i, 2}, 'm');
    plot(phi_knee_S_sit_interp{i, 1} * 180 / pi, -M_knee_S_sit_mean{i, 1}, 'b');
    plot(phi_knee_P_sit_interp{i, 1} * 180 / pi, -M_knee_P_sit_mean{i, 1}, 'r');
    xlabel(['Knee angle (deg)']);
    ylabel('Knee moment (Nm)'); 
    xticks([0,20,40,60,80,100]);
    yticks([0,50,100,150]);
    xlim([0, 100]);
    ylim([-20, 150]);
end
lh = legend('Sound Leg with Support','Prosthetic Leg with Support','Sound Leg w/o Support','Prosthetic Leg w/o Support');
set(lh, 'FontSize', 12);
set(lh, 'Position', [0.4227    0.1658    0.1930    0.1225]);
set(gcf, 'Position', get(0, 'Screensize'));
%saveas(gcf, 'MeanTorqueCurvesSit', 'epsc');
