% Function to plot bar charts with error bars representing the mean stand up and sit down times
% for the different subject and experimental conditions
% Written by A. Pennycott, February 2023

function PlotMeanStandSitTimes(meanStandTime, meanSitTime, stdStandTime, stdSitTime)

figure
hold on;
b = bar(meanStandTime, 'grouped');
drawnow;
[ngroups,nbars] = size(meanStandTime);

x = GetXCoord(b, nbars, ngroups);

errorbar(x', meanStandTime, -stdStandTime, stdStandTime, 'color', 'k', 'linewidth', 0.75, 'linestyle', 'none');
set(gca, 'ytick', (0:1:5)');
set(gca, 'xticklabel', {'S1', 'S2', 'S3', 'S4', 'S5', 'S6', 'S7'});
xlabel('Subject');
ylabel('Time (s)');
title('Mean stand-up times with and without ERiK support');    
legend('No support', 'With support');
set(gca, 'box','off');
set(gcf, 'Position', get(0, 'Screensize'));
%saveas(gcf, 'MeanStandTimes', 'epsc');

%% sitting
figure
hold on;
b = bar(meanSitTime, 'grouped');
drawnow;

[ngroups,nbars] = size(meanSitTime);

x = GetXCoord(b, nbars, ngroups);

errorbar(x', meanSitTime, -stdSitTime, stdSitTime, 'color', 'k', 'linewidth', 0.75, 'linestyle', 'none');
set(gca, 'ytick', (0:2:10)');
set(gca, 'xticklabel', {'S1', 'S2', 'S3', 'S4', 'S5', 'S6', 'S7'});
xlabel('Subject');
ylabel('Time (s)');
title('Mean sit-down times with and without ERiK support');   
legend('No support', 'With support');
set(gca, 'box','off');
set(gcf, 'Position', get(0, 'Screensize'));
%saveas(gcf, 'MeanSitTimes', 'epsc');

function x = GetXCoord(b, nbars, ngroups)
% Get the x coordinate of the bars
x = zeros(nbars, ngroups);
for i = 1:nbars
    xOff = b(i).XOffset;
    x(i,:) = b(i).XData;
    for j = 1:size(x, 2)
        x(i, j) = x(i, j) + xOff;
    end
end