%% Function to create joint angles from marker data
% Written by Simon van der Helm, April 2019

function [angles, theta, FA, FC, FD, MLA, MRA, MLK, MRK, MLH, MRH, cut, LFH_all, RFH_all] = makeJointAnglesandMoments(filelocation, data, Force,varargin)
bVideo=0;%create a video or not
if isempty(varargin)
    coordSys = MakeLocalCoord(data,'All'); % Check if the coordinate system was provided
    X = localCoord(data,coordSys);
    bAnimate = 0;
elseif length(varargin)==1
    X = varargin{1}; % If local marker coordinates were provided
    bAnimate = 0;
else
    X = varargin{1}; % Local marker coordinates were provided
    bAnimate = varargin{2}; % Animation boolean provided
end

segment_labels = {'Pelvis','LThigh';'Pelvis','RThigh';... % Segments
    'LThigh','LShank';'RThigh','RShank';...
    'LShank','LFoot';'RShank','RFoot'};
%HV: renamed "joints" to "segments", which appears more fitting here.

num_segments=size(segment_labels,1);

alpha = zeros(num_segments,size(data.Data,3)); % Euler Angles - X Y Z
beta = zeros(num_segments,size(data.Data,3));
gamma = zeros(num_segments,size(data.Data,3));
theta = zeros(3,size(data.Data,3)); % Trunk Angle
FA = zeros(3,size(data.Data,3)); % Force Plate Data
FC = zeros(3,size(data.Data,3));
FD = zeros(3,size(data.Data,3));
MLK = zeros(3,size(data.Data,3)); % Joint Moments
MRK = zeros(3,size(data.Data,3));
armLK = zeros(1,size(data.Data,3));
armRK = zeros(1,size(data.Data,3));
MLA = zeros(3,size(data.Data,3));
MRA = zeros(3,size(data.Data,3));
MLH = zeros(3,size(data.Data,3));
MRH = zeros(3,size(data.Data,3));

LFH_all = zeros(size(data.Data,3),3); % left femoral head
RFH_all = zeros(size(data.Data,3),3);   % right femoral head

labeled_data_sample = struct('Data',{data.Data(:,:,1)},'Labels',{data.Labels});
CoordSys = findLocalCoord(labeled_data_sample,X);

for i_segment = 1:num_segments
    if isfield(CoordSys,['R_',segment_labels{i_segment,1}]) && isfield(CoordSys,['R_',segment_labels{i_segment,2}])
        % Calculate the Euler Angles
        Rtemp = inv(CoordSys.(['R_',segment_labels{i_segment,1}]))*CoordSys.(['R_',segment_labels{i_segment,2}]);
        beta(i_segment,1) = asin(-Rtemp(3,1));
        %alpha(i_segment,1) = asin(Rtemp(2,1)/cos(beta(i_segment,1)));
        %
        alpha(i_segment,1) = atan2(Rtemp(2,1),Rtemp(1,1));
        
        gamma(i_segment,1) = asin(Rtemp(3,2)/cos(beta(i_segment,1)));
    else
        alpha(i_segment,1) = nan;
        beta(i_segment,1) = nan;
        gamma(i_segment,1) = nan;
    end
end

%the three force plates used are named A,C,D
% Calculate Ground Reaction Force
FA(:,1) = -Force(1).Force(:,1); % GRF chair
FC(:,1) = -Force(2).Force(:,1); %left foot
FD(:,1) = -Force(3).Force(:,1); %right foot

CoPA = Force(1).COP(:,1); % Center of Pressure chair(?)
CoPC = Force(2).COP(:,1); %left foot CoP
CoPD = Force(3).COP(:,1); %right foot CoP

if isfield(CoordSys,'O_LThigh')
    rLK2CoP = CoPC-CoordSys.O_LThigh; % Vector from left knee to left CoP
else
    rLK2CoP = nan(3,1);
end

if isfield(CoordSys,'O_RThigh')
    rRK2CoP = CoPD-CoordSys.O_RThigh; % Vector from right knee to right CoP
else
    rRK2CoP = nan(3,1);
end

MLK(:,1) = cross(rLK2CoP,FC(:,1))/1000; % Calculate knee Joint Moments
MRK(:,1) = cross(rRK2CoP,FD(:,1))/1000;
armLK(1) = norm(MLK(:,1))/norm(FC(:,1)); % Calculate Moment Arm
armRK(1) = norm(MRK(:,1))/norm(FD(:,1));

LFH = findFemoralHead('L',X,CoordSys,'local'); % Find femoral head
RFH = findFemoralHead('R',X,CoordSys,'local');

    LFH_all(1,:)=LFH';
    RFH_all(1,:)=RFH';

if ~isempty(LFH)
    rLH2CoP = CoPC-LFH; % Vector from Left Hip to CoP
else
    rLH2CoP = nan(3,1);
end

if ~isempty(RFH)
    rRH2CoP = CoPD-RFH; % Vector from Right Hip to CoP
else
    rRH2CoP = nan(3,1);
end

MLH(:,1) = cross(rLH2CoP,FC(:,1))/1000; % Calculate Hip Torque
MRH(:,1) = cross(rRH2CoP,FD(:,1))/1000;

MLH(:,1) = CoordSys.R_Pelvis\MLH(:,1); % Reset Hip Moment to local axes
MRH(:,1) = CoordSys.R_Pelvis\MRH(:,1);

if isfield(CoordSys,'O_LShank')
    rLA2CoP = CoPC-CoordSys.O_LShank; % Vector from Ankle Joint to CoP
else
    rLA2CoP = nan(3,1);
end

if isfield(CoordSys,'O_RShank')
    rRA2CoP = CoPD-CoordSys.O_RShank; % Vector from Ankle Joint to CoP
else
    rRA2CoP = nan(3,1);
end

MLA(:,1) = cross(rLA2CoP,FC(:,1))/1000; % Ankle Joint Moment
MRA(:,1) = cross(rRA2CoP,FD(:,1))/1000;

MLA(:,1) = CoordSys.R_LShank\MLA(:,1); % Reset Ankle Moment to local axes
MRA(:,1) = CoordSys.R_RShank\MRA(:,1);

iLCA = find(strcmp(data.Labels,'L-FCC')); % Find the foot coordinates
iLFM1 = find(strcmp(data.Labels,'L-FM1'));
iLFM5 = find(strcmp(data.Labels,'L-FM5'));

iRCA = find(strcmp(data.Labels,'R-FCC'));
iRFM1 = find(strcmp(data.Labels,'R-FM1'));
iRFM5 = find(strcmp(data.Labels,'R-FM5'));

iC7 = find(strcmp(data.Labels,'C7')); % Find the chest markers
iIJ = find(strcmp(data.Labels,'IJ'));

if ~isempty(iC7)
    H(:,1) = (LFH+RFH)/2; % Bottom of the trunk
    S(:,1) = (data.Data(iC7,:,1)+data.Data(iIJ,:,1))/2; % Shoulder Location
    theta(1,1) = atan((S(1,1)-H(1,1))/(S(3,1)-H(3,1))); % Trunk Angles (not Euler angles)
    theta(2,1) = atan((S(2,1)-H(2,1))/(S(3,1)-H(3,1)));
    theta(3,1) = atan((S(2,1)-H(2,1))/(S(1,1)-H(1,1)));
end

if bAnimate == 1 % Animate the results
    
    minX = min(min(data.Data(:,1,:)));
    minY = min(min(data.Data(:,2,:)));
    minZ = min(min(data.Data(:,3,:)));
    maxX = max(max(data.Data(:,1,:)));
    maxY = max(max(data.Data(:,2,:)));
    %    maxZ = max(max(data.Data(:,3,:)));
    maxZ = 1500; % RESET THIS!!    
    
    for i_xyz = 1:3%loop through the x,y, and z coordinates of the segments
        %HV: reverse-engineered and renamed variable "tempdata" to
        %"joint_centers".
        try
            % Check if all joint locations are available
            joint_centers(i_xyz,:) = [CoordSys.O_LShank(i_xyz),CoordSys.O_LThigh(i_xyz),LFH(i_xyz),RFH(i_xyz),CoordSys.O_RThigh(i_xyz),CoordSys.O_RShank(i_xyz)];
        catch
            % Find which joint location is not available
            missPoint = find(~isfield(CoordSys,{'O_LShank','O_LThigh','O_RThigh','O_RShank'}));
            % Set all available joints
            for i_member=[1,2,5,6]%these could be missing
                if ismember(i_member,missPoint)
                    joint_centers(i_xyz,i_member) = NaN;
                else
                    joint_centers(i_xyz,i_member) = CoordSys.O_LShank(i_xyz);
                end
            end
            joint_centers(i_xyz,3) = LFH(i_xyz);%these are never missing
            joint_centers(i_xyz,4) = RFH(i_xyz);
        end
    end
    
    fig = figure(200);
    clf
    % Plot all markers
    p1 = plot3(data.Data(:,1,1),data.Data(:,2,1),data.Data(:,3,1),'b*'); hold on; axis equal;
    % Plot Feet
    p2 = plot3(data.Data([iLCA,iLFM1,iLFM5,iLCA],1,1),data.Data([iLCA,iLFM1,iLFM5,iLCA],2,1),data.Data([iLCA,iLFM1,iLFM5,iLCA],3,1),'Color','r','linewidth',2);
    p3 = plot3(data.Data([iRCA,iRFM1,iRFM5,iRCA],1,1),data.Data([iRCA,iRFM1,iRFM5,iRCA],2,1),data.Data([iRCA,iRFM1,iRFM5,iRCA],3,1),'Color','r','linewidth',2);
    
%     %plot metatarsals, to check correct labeling
%     Lmet1 = plot3(data.Data(iLFM1,1,1),data.Data(iLFM1,2,1),data.Data(iLFM1,3,1),'c*');
%     Rmet1 = plot3(data.Data(iRFM1,1,1),data.Data(iRFM1,2,1),data.Data(iRFM1,3,1),'m*');    
    
    % Plot  the legs
    p4 = plot3(joint_centers(1,:),joint_centers(2,:),joint_centers(3,:),'Color','r','linewidth',2);
    % Plot the Trunk
    if exist('S','var')
        p5 = plot3([H(1,1),S(1,1)],[H(2,1),S(2,1)],[H(3,1),S(3,1)],'Color','r','linewidth',2);
    end
    
    % Plot the GRFs
    q1 = quiver3(CoPA(1),CoPA(2),CoPA(3),FA(1,1),FA(2,1),FA(3,1),0,'Linewidth',2,'MaxHeadSize',2,'Color','g');
    q2 = quiver3(CoPC(1),CoPC(2),CoPC(3),FC(1,1),FC(2,1),FC(3,1),0,'Linewidth',2,'MaxHeadSize',2,'Color','g');
    q3 = quiver3(CoPD(1),CoPD(2),CoPD(3),FD(1,1),FD(2,1),FD(3,1),0,'Linewidth',2,'MaxHeadSize',2,'Color','g');
    axis([minX-50,maxX+50,minY-50,maxY+50,minZ-50,maxZ+50]);
    xlabel('x/mm')
    ylabel('y/mm')
    zlabel('z/mm')
    
    if bVideo
        % Open the video to save the animation
        v = VideoWriter([filelocation,'\Animation.avi']);
        v.FrameRate = 15;
        open(v);
        
        % Save the frame
        im = cell(size(data.Data,3),1);
        frame = getframe(fig);
        im{1} = frame2im(frame);
        
        % Add the frame to the video
        writeVideo(v,frame);
        
        [imind,cm] = rgb2ind(im{1},256);
        imwrite(imind,cm,[filelocation,'\Animation.gif'],'gif', 'Loopcount',inf);
    end
end

for i_sample = 2:size(data.Data,3) % Loop through the samples
    labeled_data_sample = struct('Data',{data.Data(1:length(data.Labels),:,i_sample)},'Labels',{data.Labels});
    CoordSys = findLocalCoord(labeled_data_sample,X); % Construct Local Coordinates
    for i_segment = 1:num_segments
        if isfield(CoordSys,['R_',segment_labels{i_segment,1}]) && isfield(CoordSys,['R_',segment_labels{i_segment,2}])
            Rtemp = inv(CoordSys.(['R_',segment_labels{i_segment,1}]))*CoordSys.(['R_',segment_labels{i_segment,2}]);
            beta(i_segment,i_sample) = asin(-Rtemp(3,1)); % Calculate Euler Angles
            %alpha(i_segment,i_sample) = asin(Rtemp(2,1)/cos(beta(i_segment,i_sample)));
            alpha(i_segment,i_sample) = atan2(Rtemp(2,1),Rtemp(1,1));%HV: improved to make sure angle can be below 0 (hyperextension)
            
            gamma(i_segment,i_sample) = asin(Rtemp(3,2)/cos(beta(i_segment,i_sample)));
        else
            alpha(i_segment,i_sample) = nan;
            beta(i_segment,i_sample) = nan;
            gamma(i_segment,i_sample) = nan;
        end
    end
    % Extract Forces:
    FA(:,i_sample) = -Force(1).Force(:,i_sample);
    FC(:,i_sample) = -Force(2).Force(:,i_sample);
    FD(:,i_sample) = -Force(3).Force(:,i_sample);
    
    % Extract CoPs:
    CoPA = Force(1).COP(:,i_sample);
    CoPC = Force(2).COP(:,i_sample);
    CoPD = Force(3).COP(:,i_sample);
    
    %calculate joint moments:
    if isfield(CoordSys,'O_LThigh')
        rLK2CoP = CoPC-CoordSys.O_LThigh; % Vector from Left Knee to left CoP
    else
        rLK2CoP = nan(3,1);
    end
    
    if isfield(CoordSys,'O_RThigh')
        rRK2CoP = CoPD-CoordSys.O_RThigh; % Vector from Right Knee to right CoP
    else
        rRK2CoP = nan(3,1);
    end
    
    MLK(:,i_sample) = cross(rLK2CoP,FC(:,i_sample))/1000; % Calculate Knee Moment
    MRK(:,i_sample) = cross(rRK2CoP,FD(:,i_sample))/1000;
    armLK(i_sample) = norm(MLK(:,i_sample))/norm(FC(:,i_sample)); % Calculate moment arm
    armRK(i_sample) = norm(MRK(:,i_sample))/norm(FD(:,i_sample));
    
    LFH = findFemoralHead('L',X,CoordSys,'local'); % Find the femoral head
    RFH = findFemoralHead('R',X,CoordSys,'local');
    
    LFH_all(i_sample,:)=LFH';
    RFH_all(i_sample,:)=RFH';
  
    if ~isempty(LFH)
        rLH2CoP = CoPC-LFH; % Vector from Left Hip to left CoP
    else
        rLH2CoP = nan(3,1);
    end
    
    if ~isempty(RFH)
        rRH2CoP = CoPD-RFH; % Vector from Right Hip to right CoP
    else
        rRH2CoP = nan(3,1);
    end
    
    MLH(:,i_sample) = cross(rLH2CoP,FC(:,i_sample))/1000; % Hip Moment
    MRH(:,i_sample) = cross(rRH2CoP,FD(:,i_sample))/1000;
    
    MLH(:,i_sample) = CoordSys.R_Pelvis\MLH(:,i_sample); % Transform Hip Moment to local coordinate system axes
    MRH(:,i_sample) = CoordSys.R_Pelvis\MRH(:,i_sample);
    
    if isfield(CoordSys,'O_LShank')
        rLA2CoP = CoPC-CoordSys.O_LShank; % Vector from Left Ankle to CoP
    else
        rLA2CoP = nan(3,1);
    end
    
    if isfield(CoordSys,'O_RShank')
        rRA2CoP = CoPD-CoordSys.O_RShank; % Vector from Right Ankle to CoP
    else
        rRA2CoP = nan(3,1);
    end
    
    MLA(:,i_sample) = cross(rLA2CoP,FC(:,i_sample))/1000; % Calculate Ankle Moment
    MRA(:,i_sample) = cross(rRA2CoP,FD(:,i_sample))/1000;
    
    MLA(:,i_sample) = CoordSys.R_LShank\MLA(:,i_sample); % Transform Ankle Moment to local coordinate system axes
    MRA(:,i_sample) = CoordSys.R_RShank\MRA(:,i_sample);
    
    if ~isempty(iC7) % Check if the trunk markers exist
        H(:,i_sample) = (LFH+RFH)/2; % Trunk Base
        S(:,i_sample) = (data.Data(iC7,:,i_sample)+data.Data(iIJ,:,i_sample))/2; % Trunk Top
        theta(1,i_sample) = atan((S(1,i_sample)-H(1,i_sample))/(S(3,i_sample)-H(3,i_sample))); % Trunk Angles
        theta(2,i_sample) = atan((S(2,i_sample)-H(2,i_sample))/(S(3,i_sample)-H(3,i_sample)));
        theta(3,i_sample) = atan((S(2,i_sample)-H(2,i_sample))/(S(1,i_sample)-H(1,i_sample)));
        
        if mod(i_sample,10)==1 && bAnimate == 1 % Animate results, only add every n-th sample
            for i_xyz = 1:3
                try
                    % Check if all joint locations are available
                    joint_centers(i_xyz,:) = [CoordSys.O_LShank(i_xyz),CoordSys.O_LThigh(i_xyz),LFH(i_xyz),RFH(i_xyz),CoordSys.O_RThigh(i_xyz),CoordSys.O_RShank(i_xyz)];
                catch
                    % Check which joint location is not available
                    missPoint = find(~isfield(CoordSys,{'O_LShank','O_LThigh','O_RThigh','O_RShank'}));
                    % Fill the available joint locations
                    if ismember(1,missPoint)
                        joint_centers(i_xyz,1) = NaN;
                    else
                        joint_centers(i_xyz,1) = CoordSys.O_LShank(i_xyz);
                    end
                    if ismember(2,missPoint)
                        joint_centers(i_xyz,2) = NaN;
                    else
                        joint_centers(i_xyz,2) = CoordSys.O_LThigh(i_xyz);
                    end
                    if ismember(3,missPoint)
                        joint_centers(i_xyz,5) = NaN;
                    else
                        joint_centers(i_xyz,5) = CoordSys.O_RThigh(i_xyz);
                    end
                    if ismember(4,missPoint)
                        joint_centers(i_xyz,6) = NaN;
                    else
                        joint_centers(i_xyz,6) = CoordSys.O_RShank(i_xyz);
                    end
                    joint_centers(i_xyz,3) = LFH(i_xyz);
                    joint_centers(i_xyz,4) = RFH(i_xyz);
                end
            end
                        
            % Update Markers
            set(p1,'XData',data.Data(:,1,i_sample),'YData',data.Data(:,2,i_sample),'ZData',data.Data(:,3,i_sample));
            % Update Feet
            set(p2,'XData',data.Data([iLCA,iLFM1,iLFM5,iLCA],1,i_sample),'YData',data.Data([iLCA,iLFM1,iLFM5,iLCA],2,i_sample),'ZData',data.Data([iLCA,iLFM1,iLFM5,iLCA],3,i_sample));
            set(p3,'XData',data.Data([iRCA,iRFM1,iRFM5,iRCA],1,i_sample),'YData',data.Data([iRCA,iRFM1,iRFM5,iRCA],2,i_sample),'ZData',data.Data([iRCA,iRFM1,iRFM5,iRCA],3,i_sample));                     
            
            % Update Legs
            set(p4,'XData',joint_centers(1,:),'YData',joint_centers(2,:),'ZData',joint_centers(3,:));
            % Update Trunk
            if exist('S','var')
                set(p5,'XData',[H(1,i_sample),S(1,i_sample)],'YData',[H(2,i_sample),S(2,i_sample)],'ZData',[H(3,i_sample),S(3,i_sample)]);
            end
            %Update GRFs
            set(q1,'XData',CoPA(1),'YData',CoPA(2),'ZData',CoPA(3),'UData',FA(1,i_sample),'VData',FA(2,i_sample),'WData',FA(3,i_sample));
            set(q2,'XData',CoPC(1),'YData',CoPC(2),'ZData',CoPC(3),'UData',FC(1,i_sample),'VData',FC(2,i_sample),'WData',FC(3,i_sample));
            set(q3,'XData',CoPD(1),'YData',CoPD(2),'ZData',CoPD(3),'UData',FD(1,i_sample),'VData',FD(2,i_sample),'WData',FD(3,i_sample));
            drawnow
            
            if bVideo
                % Save Frame
                frame = getframe(fig);
                im{i_sample} = frame2im(frame);
                
                % Add frame to video
                writeVideo(v,frame);
                
                [imind,cm] = rgb2ind(im{i_sample},256);
                imwrite(imind,cm,[filelocation,'\Animation.gif'],'gif','WriteMode','append','DelayTime',0.03);
            end
        end
        
    end
    
end
if bAnimate == 1 % Animate the results
    close(fig);
end
if bVideo
    close(v);
end

if any(theta)
    cut.success=1;
    % cutAnimations function with trunk angles available
    if bVideo
        [cut.times, cut.names] = CutAnimations(alpha, theta(1,:), filelocation, im); % Returns the times and names of the intervals
    else
        [cut.times, cut.names] = CutAnimations(alpha, theta(1,:), filelocation); % Returns the times and names of the intervals
    end
else
    disp('skipping the cut')
    cut.success=0;
end

angles(:,:,1) = alpha;
angles(:,:,2) = beta;
angles(:,:,3) = gamma;

function [times, names] = CutAnimations(alpha,theta,filelocation,varargin)
if isempty(varargin)
    exist_im = 0;%no video being created
elseif length(varargin) == 1
    im = varargin{1}; % If video is being created
    exist_im=1;
else
    disp('too many arguments')
end

sSit = 1; % Number of successful sit down transitions
sStand = 1; % Number of successful stand up transitions
fSit = 1; % Number of failed sit down transitions
fStand = 1; % Number of failed stand up transitions

sitTheta = findQuietSit(theta,alpha); % Find the quiet sitting angle
standTheta = findQuietStand(theta,alpha); % Find the quiet standing angle
sitThresh = 0.02*(max(theta)-sitTheta)+sitTheta; % Find the threshold that marks the start of a standing up transition
[sitFlex,sitEx] = detectThreshold(theta,sitThresh,[1,-1]); % Detect threshold crossings
standThresh = 0.02*(max(theta)-standTheta)+standTheta; % Find the threshold that marks the start of a sitting down transition
[standFlex,standEx] = detectThreshold(theta,standThresh,[1,-1]); % Detect threshold crossings

sSitPos = (alpha(3,sitFlex)>pi/4|alpha(4,sitFlex)>pi/4); % Find downwards threshold crossings with seated position
eSitPos = (alpha(3,sitEx)>pi/4|alpha(4,sitEx)>pi/4); % Find upwards crossings with seated position

sStandPos = and(alpha(3,standFlex)<pi/4,alpha(4,standFlex)<pi/4); % Find downwards threshold crossings with standing position
eStandPos = and(alpha(3,standEx)<pi/4,alpha(4,standEx)<pi/4); % Find upwards crossing with standing position

[temp,sI] = sort([sitFlex(sSitPos),standFlex(sStandPos)]); % Sort the start of the transitions

if length(temp) >= 2*str2double(filelocation(find(filelocation=='\',1,'last')+1)) % Check if the number of detected transitions is at least the number defined in the trials
    times(1,:) = temp;
    correct=1;
else
    correct = 0;
    times(1,:) = 1;    
end

if correct == 1
    [temp,eI] = sort([sitEx(eSitPos),standEx(eStandPos)]); % Sort the end of transitions
    
    tsPos = [ones(1,length(find(sSitPos))),zeros(1,length(find(sStandPos)))]; % Starting positions (1 is sitting, 0 is standing)
    tsPos = tsPos(sI);
    tePos = [ones(1,length(find(eSitPos))),zeros(1,length(find(eStandPos)))]; % Ending positions (1 is sitting, 0 is standing)
    tePos = tePos(eI);
    
    if temp(1)<times(1,1) % If the trial starts with an endpoint, remove the endpoint
        temp(1) = [];
        tePos(1) = [];
    end
    
    if length(temp)~=size(times,2) % If the number of endpoints is not the same as the number of starting points
        if length(temp)>size(times,2) % If the number of endpoints is greater
            crossings = [times(1,:);temp(1:size(times,2))]; % All start and endpoints
            crossings = [crossings(:);temp(size(times,2)+1:end)'];
            if issorted(crossings) % Check if they are sorted
                times(2,:) = temp(1:size(times,2)); % If they are sorted, remove the additional endpoints
                ePos = tePos(1:size(times,2));
            else
                mat = [times(1,:),length(theta)+1]; % Starting times and the end of the trial
                for i = 1:size(times,2)
                    times(2,i) = temp(find(temp<mat(i+1),1,'last')); % Find the endpoints that are furthest from the starting point
                    ePos(i) = tePos(find(temp<mat(i+1),1,'last'));
                end
                
            end
            sPos = tsPos;
        else
            crossings = [times(1,1:length(temp));temp]; % All start and endpoints
            crossings = [crossings(:);times(1,length(temp)+1:end)'];
            if issorted(crossings) % Check if sorted
                if length(temp)+1==size(times,2) % If there is one endpoint missing
                    times(2,:) = [temp,length(theta)]; % Add the end of trial as endpoint
                    ePos = [tePos,(alpha(3,end)>pi/4|alpha(4,end)>pi/4)];
                    sPos = tsPos;
                else
                    times(:,length(temp)+2:end) = []; % Remove all additional starting points
                    sPos = tsPos(1:length(temp)+1);
                    times(2,:) = [temp,length(theta)]; % Add the end of trial as endpoint
                    ePos = [tePos,(alpha(3,end)>pi/4|alpha(4,end)>pi/4)];
                end
            else
                temp = [temp,length(theta)]; % Add the end of trial to the endpoints
                mat = [0,temp];
                for i = 1:length(temp)
                    betw = find(and(times(1,:)>mat(i),times(1,:)<mat(i+1))); % Find the starting points between endpoints
                    if length(betw)>1 % If multiple starting points between endpoints
                        times(:,betw(2:end)) = []; % Remove additional starting points
                        tsPos(betw(2:end)) = [];
                    elseif length(betw)<1 % If no starting points between endpoints
                        temp(i) = nan; % Remove an endpoint
                        tePos(i) = nan;
                    end
                end
                times(2,:) = temp(~isnan(temp)); % Remove the endpoints
                ePos = tePos(~isnan(tePos));
                sPos = tsPos;
            end
        end
    else % If the number of start and endpoints is the same
        sPos = tsPos; % Save the start and endpoints
        ePos = tePos;
        times(2,:) = temp;
    end
    
    times(3,:) = zeros(size(times(1,:)));
    names = cell(1,size(times,2));
    
    
    
    % Check whether transition was successful or failed, and sit down or stand
    % up
    for i = 1:size(times,2)
        if sPos(i) == 1
            if ePos(i) == 1
                names{i} = ['FailedStand',num2str(fStand)];
                fStand = fStand+1;
                times(3,i) = 3;
            else
                names{i} = ['SuccessStand',num2str(sStand)];
                sStand = sStand+1;
                times(3,i) = 1;
            end
        else
            if ePos(i) == 1
                names{i} = ['SuccessSit',num2str(sSit)];
                sSit = sSit+1;
                times(3,i) = 2;
            else
                names{i} = ['FailedSit',num2str(fSit)];
                fSit = fSit+1;
                times(3,i) = 4;
            end
        end
        
        % Create transition folder, if it does not exist
        if ~exist([filelocation,'\',names{i}], 'dir')
            mkdir(filelocation,names{i})
        end
        
        if exist_im
            % Save the part of the animation of the specific transition
            [imind,cm] = rgb2ind(im{times(1,i)-mod(times(1,i),3)+1},256);
            imwrite(imind,cm,[filelocation,'\',names{i},'\',names{i},'.gif'],'gif', 'Loopcount',inf);
            for j = times(1,i)-mod(times(1,i),3)+4:3:times(2,i)
                [imind,cm] = rgb2ind(im{j},256);
                imwrite(imind,cm,[filelocation,'\',names{i},'\',names{i},'.gif'],'gif','WriteMode','append','DelayTime',0.03);
            end
        end
    end
    
else
    times=[];
    names = [];
end

function [theta] = findQuietSit(Theta,alpha)

thresh = 0.005; % Threshold
n = 100; % Number of points
step = 10; % Stepsize

p = 1; % Evaluated section start point
stdTheta = 100; % Standard Deviation

% Find a section in which the standard deviation of the trunk angle is
% smaller than threshold to return the quiet sitting trunk angle
while (stdTheta > thresh && p<length(Theta)-100)  
    if alpha(3,ceil(p+n/2)) > pi/4 || alpha(4,ceil(p+n/2)) > pi/4 % Both Knee angles larger than pi/4 means sitting position
        stdTheta = std(Theta(p:p+n)); % Calculate standard deviation of the points in the section
        theta = mean(Theta(p:p+n)); % Calculate the mean of the points in the section
    end
    p = p+step; % Move the section by stepsize points
end

% If no section without trunk movement was found, return a value of 100
if (stdTheta > thresh)
    theta = 100;
end

function [theta] = findQuietStand(Theta,alpha)

thresh = 0.005; % Threshold
n = 100; % Number of points
step = 10; % Stepsize

p = 1; % Starting point
stdTheta = 100; % Initialize standard deviation

% Find a section in which the standard deviation is smaller than the
% threshold, to find the quiet standing trunk angle
while (stdTheta > thresh && p < length(Theta)-100)
    if alpha(3,ceil(p+n/2)) < pi/4 && alpha(4,ceil(p+n/2)) < pi/4 % Both knee angles smaller than pi/4 means standing
        stdTheta = std(Theta(p:p+n)); % Calculate standard deviation
        theta = mean(Theta(p:p+n)); % Calculate mean trunk angle
    end
    p = p+step; % Move the section by step samples
end

% If quiet standing was not found, return 100
if (stdTheta > thresh)
    theta = 100;
end

%% Function to detect threshold crossings in Data
% Written by Simon van der Helm, April 2019
%
% [Index] = detectThreshold(data,T,direction)
% Index is an array with the occurrance of threshold crossings. data is
% contains the data, which crosses the threshold. If data is a matrix, the
% indices will be for each column. T contains the threshold(s). T is either
% a scalar, or a 1xN vector, where N is the number of columns in data.
% direction shows the direction of the threshold crossing, with '1' being
% upwards, '-1' being downwards and '0' being both ways. direction is
% either a scalar, or a 1xN vector, where N is the number of columns in
% data. Alternatively, if data is a vector, direction could also be a
% 2x1 vector containing [1,-1]. This will return positive and negative
% crossings in different columns in Index.


function [varargout] = detectThreshold(data,T,direction)

if any(size(data)==1) && any(size(T)~=1) % Check Threshold dimensions
    error('Incorrect dimension Threshold Vector');
elseif sum(size(T)>1)>1
    error('Incorrect dimension Threshold Vector');
elseif max(size(T))~=size(data,2) && max(size(T))~=1
    error('Incorrect dimension Threshold Vector');
end

if any(size(data)==1) && any(size(direction)>2) % Check Data dimensions
    error('Incorrect dimension direction vector');
elseif sum(size(direction)>1)>1
    error('Incorrect dimension direction vector');
elseif max(size(direction))~=size(data,2) && max(size(direction))>2
    error('Incorrect dimension direction vector');
end

if size(data,1)==1 % Make data a column vector
    data = data';
end



if sum(size(data)>1)>1
    if max(size(T))==1
        T = repmat(T,1,size(data,2)); % If Data is matrix, make Threshold a row vector
    end
    if max(size(direction))==1
        direction = repmat(direction,1,size(data,2)); % If Data is a matrix, make direction a row vector
    end
else
    if max(size(direction))==2 % If Data is a vector and direction is [2x1], repeat the data columns to find crossings in both directions
        data = repmat(data,1,2);
        T = repmat(T,1,2);
    end
end

varargout = cell(1,size(data,2)); % Initialize output

for i = 1:size(data,2)
    bVec = (data(:,i)>T(i))'; % Find values above the threshold
    if direction(i)<0 % If direction is negative
        varargout{i} = strfind(bVec,[1,0]); % Find instances when data goes from above threshold to below threshold
    elseif direction(i)>0 % If direction is positive
        varargout{i} = strfind(bVec,[0,1]); % Find instances when data goes from below threshold to above threshold
    else
        varargout{i} = sort([strfind(bVec,[1,0]),strfind(bVec,[0,1])]); % Find instances when data crosses threshold in both directions
    end
end

if length(varargout)>nargout
    varargout{1} = varargout; % If the requested output is a single matrix, return the cell matrix
end
