%% Function to make a Coordinate System for the Pelvis segment
% Written by Simon van der Helm, April 2019

function [O,R] = makePelvisCoord(data,labels)

LASIS = data(strcmp(labels,'L-IAS'),1:3)'; % Left Anterior Superior Iliac Spine
RASIS = data(strcmp(labels,'R-IAS'),1:3)'; % Right Anterior Superior Iliac Spine
LPSIS = data(strcmp(labels,'L-IPS'),1:3)'; % Left Posterior Superior Iliac Spine
RPSIS = data(strcmp(labels,'R-IPS'),1:3)'; % Right Posterior Superior Iliac Spine

O = (LASIS-RASIS)/2+RASIS; % Place the origin
z = (LASIS-RASIS)/norm(LASIS-RASIS); % Choose the z-axis
M = (LPSIS-RPSIS)/2+RPSIS; % Find the middle between the Posterior Superior Iliac Spines
ytemp = (O-M)/norm(O-M); % Temporary y-axis
x = cross(ytemp,z); % x-axis perpendicular to both y and z axes
x = x/norm(x); % Normalize
y = cross(z,x); % y-axis perpendicular to both x and z axis
y = y/norm(y); % Normalize

R = [x,y,z]; % Build the rotation matrix